@@ -10,11 +10,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -137,12 +132,12 @@ int pageposition(short libmode, XButtonEvent *event, int mode)
    }
 }
 
-/*--------------------------------------------------------*/
-/* Find the number of other pages linked to the indicated */
-/* page (having the same filename, and ignoring empty	  */
-/* pages).  result is the total number of pages in the	  */
-/* output file.						  */
-/*--------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Find the number of other pages linked to the		*/
+/* indicated page (having the same filename, and	*/
+/* ignoring empty pages).  result is the total number	*/
+/* of pages in the output file.				*/
+/*------------------------------------------------------*/
 
 short pagelinks(int page)
 {
@@ -159,6 +154,94 @@ short pagelinks(int page)
    return count;
 }
 
+/*------------------------------------------------------*/
+/* This is an expanded version of pagelinks() (above),	*/
+/* to deal with the separate issues of independent top-	*/
+/* level schematics and subcircuits.  For the indicated	*/
+/* page, return a list of pages depending on the mode:	*/
+/*							*/
+/* mode = INDEPENDENT: independent top-level pages	*/
+/* mode = DEPENDENT: dependent pages (subcircuits)	*/
+/* mode = PAGE_DEPEND: subcircuits of the current page,	*/
+/* mode = TOTAL_PAGES: independent pages + subcircuits  */
+/* mode = ALL_PAGES: all pages in xcircuit		*/
+/*							*/
+/* The list is the size of the number of pages, and	*/
+/* entries corresponding to the requested mode are set	*/
+/* nonzero (the actual number indicates the number of	*/
+/* references to the page, which may or may not be	*/
+/* useful to know).					*/
+/*							*/
+/* It is the responsibility of the calling routine to	*/
+/* free the memory allocated for the returned list.	*/
+/*------------------------------------------------------*/
+
+short *pagetotals(int page, short mode)
+{
+   int i;
+   short *counts, *icount, *result;
+
+   if (xobjs.pagelist[page]->pageinst == NULL) return;
+
+   counts = (short *)malloc(xobjs.pages * sizeof(short));
+   icount = (short *)malloc(xobjs.pages * sizeof(short));
+   for (i = 0; i < xobjs.pages; i++) {
+      *(counts + i) = 0;
+      *(icount + i) = 0;
+   }
+
+   /* Find all the subcircuits of this page */
+
+   if (mode != ALL_PAGES)
+      findsubschems(page, xobjs.pagelist[page]->pageinst->thisobject, 0, counts);
+
+   /* Check independent entries (top-level pages which are not	*/
+   /* subcircuits of another page).  Set the counts entry to -1	*/
+   /* to mark each independent page.				*/
+
+   if (mode != PAGE_DEPEND)
+      for (i = 0; i < xobjs.pages; i++)
+         if (xobjs.pagelist[i]->pageinst != NULL)
+	    if (xobjs.pagelist[i]->pageinst->thisobject->parts > 0)
+	    {
+	       if (mode == ALL_PAGES)
+		  (*(counts + i)) = 1;
+	       else
+	       {
+	          if ((i == page) || (!strcmp(xobjs.pagelist[i]->filename,
+			xobjs.pagelist[page]->filename)))
+	             if (*(counts + i) == 0)
+		        (*(icount + i))++;
+	       }
+	    }
+
+   /* Check other dependent entries (top-level pages which are 	*/
+   /* subcircuits of any independent page).			*/
+
+   if ((mode == DEPENDENT) || (mode == TOTAL_PAGES))
+   {
+      for (i = 0; i < xobjs.pages; i++)
+	 if ((i != page) && (*(icount + i) > 0))
+	    findsubschems(i, xobjs.pagelist[i]->pageinst->thisobject, 0, counts);
+   }
+
+   if (mode == INDEPENDENT)
+   {
+      free((char *)counts);
+      return icount;
+   }
+   else
+   {
+      if (mode == TOTAL_PAGES) {	/* merge dependent and independent */
+	 for (i = 0; i < xobjs.pages; i++)
+	    if (*(icount + i) > 0)
+	       (*(counts + i))++;
+      }
+      free((char *)icount);
+      return counts;
+   }
+}
+
 /*------------------------------------------------------*/
 /* Test whether an object is a page, and return the	*/
 /* page number if it is.  Otherwise, return -1.		*/
@@ -568,9 +651,9 @@ void composelib(short mode)
 
    reset(libpage, NORMAL);
 
-   /* Return if library defines no objects */
+   /* Return if library defines no objects or virtual instances */
 
-   if (xobjs.userlibs[mode - LIBRARY].number == 0) return;
+   if (xobjs.userlibs[mode - LIBRARY].instlist == NULL) return;
 
    /* Find the Helvetica font for use in labeling the objects */
 
