@@ -347,11 +347,21 @@ ULPARAM_MODE, UIPARAM_MODE
 #endif
 
 /*----------------------------------------------------------------------*/
-/* Save types								*/
+/* Save types.	This list incorporates "ALL_PAGES", below.		*/
 /*----------------------------------------------------------------------*/
 
-#define CURRENT_PAGE	0
-#define ALL_PAGES	1
+#define CURRENT_PAGE	0	/* Current page + all associated pages	*/
+#define NO_SUBCIRCUITS	1	/* Current page w/o subcircuit pages	*/
+
+/*----------------------------------------------------------------------*/
+/* Modes used when ennumerating page totals.				*/
+/*----------------------------------------------------------------------*/
+
+#define INDEPENDENT	0
+#define DEPENDENT	1
+#define TOTAL_PAGES	2
+#define PAGE_DEPEND	3
+#define ALL_PAGES	4
 
 /*----------------------------------------------------------------------*/
 /* Color scheme styles (other than NORMAL)				*/
@@ -980,7 +990,7 @@ enum {
    XCF_Edit_Next, XCF_Attach, XCF_Next_Library, XCF_Library_Directory,
    XCF_Library_Move, XCF_Library_Copy, XCF_Library_Edit, XCF_Library_Delete,
    XCF_Library_Duplicate, XCF_Library_Hide, XCF_Library_Virtual,
-   XCF_Page_Directory, XCF_Library_Pop,
+   XCF_Page_Directory, XCF_Library_Pop, XCF_Virtual,
    XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
    XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
